@@ -29,7 +29,7 @@ sub new {
     return $self;
 }
 
-sub create_virtualos {
+sub create {
     my $self = shift;
 
     # Templates in ezjail are 'flavours' or archives
@@ -104,79 +104,3 @@ sub _get_jails_root {
 
 __END__
 
-=head1 NAME
-
-Provision::Unix::VirtualOS::FreeBSD::Ezjail - 
-
-=head1 VERSION
-
-Version 0.08
-
-=head1 SYNOPSIS
-
-Quick summary of what the module does.
-
-Perhaps a little code snippet.
-
-    use Provision::Unix::VirtualOS::FreeBSD::Ezjail;
-
-    my $foo = Provision::Unix::VirtualOS::FreeBSD::Ezjail->new();
-    ...
-
-
-=head1 FUNCTIONS
-
-=head2 function1
-
-=cut
-
-=head1 AUTHOR
-
-Matt Simerson, C<< <matt at tnpi.net> >>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to C<bug-unix-provision-virtualos at rt.cpan.org>, or through the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Provision-Unix>.  I will be notified, and then you'll automatically be notified of progress on your bug as I make changes.
-
-
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc Provision::Unix::VirtualOS::FreeBSD::Ezjail
-
-
-You can also look for information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Provision-Unix>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/Provision-Unix>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/Provision-Unix>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/Provision-Unix>
-
-=back
-
-
-=head1 ACKNOWLEDGEMENTS
-
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2008 Matt Simerson, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-=cut
